@@ -374,6 +374,19 @@ typedef enum {
   GST_QOS_TYPE_THROTTLE        = 2
 } GstQOSType;
 
+/**
+ * GstGapFlags:
+ * @GST_GAP_FLAG_MISSING_DATA: The #GST_EVENT_GAP signals missing data,
+ *    for example because of packet loss.
+ *
+ * The different flags that can be set on #GST_EVENT_GAP events. See
+ * gst_event_set_gap_flags() for details.
+ *
+ * Since: 1.20
+ */
+typedef enum {
+  GST_GAP_FLAG_MISSING_DATA = (1<<0),
+} GstGapFlags;
 
 /**
  * GstEvent:
@@ -557,6 +570,14 @@ void            gst_event_parse_gap             (GstEvent     * event,
                                                  GstClockTime * timestamp,
                                                  GstClockTime * duration);
 
+GST_API
+void            gst_event_set_gap_flags           (GstEvent    * event,
+                                                   GstGapFlags   flags);
+
+GST_API
+void            gst_event_parse_gap_flags         (GstEvent    * event,
+                                                   GstGapFlags * flags);
+
 /* Caps events */
 
 GST_API
